netutil

package
v0.9.4 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Jul 20, 2026 License: GPL-3.0 Imports: 7 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func ActivationListeners

func ActivationListeners() (lns map[string]net.Listener, err error)

ActivationListeners builds a map of addresses to listeners that were passed during systemd activation

func GetListener

func GetListener(socketPath string, listenerMap map[string]net.Listener) (net.Listener, error)

GetListener tries to get a listener for the given socket path from the listener map, and if it fails it tries to set it up directly.

Types

This section is empty.

Jump to