ctlcmd

package
v0.9.4 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Jul 20, 2026 License: GPL-3.0 Imports: 11 Imported by: 0

Documentation

Overview

Package ctlcmd contains the various workshopctl subcommands.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func Run

func Run(context *hookstate.Context, args []string, uid uint32) (stdout, stderr []byte, err error)

Run runs the requested command.

Types

type ForbiddenCommandError

type ForbiddenCommandError struct {
	Message string
}

ForbiddenCommandError conveys that a command cannot be invoked in some context

func (ForbiddenCommandError) Error

func (f ForbiddenCommandError) Error() string

type MissingContextError

type MissingContextError struct {
	// contains filtered or unexported fields
}

func (*MissingContextError) Error

func (e *MissingContextError) Error() string

type UnsuccessfulError

type UnsuccessfulError struct {
	ExitCode int
}

UnsuccessfulError carries a specific exit code to be returned to the client.

func (UnsuccessfulError) Error

func (e UnsuccessfulError) Error() string

Jump to