syscheck

package
v0.9.4 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Jul 20, 2026 License: GPL-3.0 Imports: 1 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func CheckSystem

func CheckSystem() error

CheckSystem ensures that the system is capable of running workshopd.

An error with details is returned if some check fails.

func RegisterCheck

func RegisterCheck(f func() error)

Types

This section is empty.

Jump to