Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
var ( // base directory inside a workshop WorkshopBaseDir = defaultBaseDir // Directory for mounted binaries (i.e. workshopctl) WorkshopGuestBinDir = filepath.Join(WorkshopBaseDir, "bin") // SDKs directory to install an SDK in a workshop WorkshopSdksDir = filepath.Join(WorkshopBaseDir, "sdk") // Base directory for the state storage WorkshopStateDir = filepath.Join(WorkshopBaseDir, "state") // Run directory inside workshop WorkshopRunDir = filepath.Join(WorkshopBaseDir, "run") // Path to the daemon's unix socket as seen from inside a workshop. The // host daemon's untrusted socket is proxied to this fixed path so that // workshopctl and hooks always find it regardless of the daemon's host // socket name (which varies, e.g. under "go tool try"). WorkshopSocketPath = filepath.Join(WorkshopRunDir, "workshop.socket") // Directory for actions inside workshop WorkshopActionsDir = filepath.Join(WorkshopRunDir, "actions") // Cache directory for deb packages AptCacheDir = "/var/cache/apt/archives" )
Variables for paths inside a workshop
var ( // Base directory for workshopd BaseDir string // Cache directory for workshopd CacheDir string // Path for workshopctl executable WorkshopCtlPath string // The directory to store downloaded base images and associated metadata BaseDownloads string // The directory to store downloaded SDKs SdkDownloads string // Path to the daemon's unix socket SocketPath string // State lock file WorkshopStateLockFile string // Base for the XDG runtime directory of a host user XdgRuntimeDirBase string // Run directory WorkshopdRunDir string // Locks directory WorkshopdLocksDir string // SSH keys WorkshopSSHDir string // Certificates WorkshopTlsDir string )
Variables for workshopd (host paths)
Functions ¶
func CreateDirs ¶
func CreateDirs() error
func SetCacheDir ¶
func SetCacheDir(cachedir string)
func SetRootDir ¶
func SetRootDir(rootdir string)
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.