Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func CheckSystem ¶
func CheckSystem() error
CheckSystem ensures that the system is capable of running workshopd.
An error with details is returned if some check fails.
func RegisterCheck ¶
func RegisterCheck(f func() error)
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.